<?php
/**
 * @package JoomlaPack
 * @subpackage Installer
 * @copyright Copyright (C) 2009 JoomlaPack Developers. All rights reserved.
 * @author Nicholas K. Dionysopoulos - http://www.dionysopoulos.me
 * @version 4.0
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL v3 or later
 *
 * JoomlaPack Installer 4 Logic: Database restoration
 * Based on BigDump by Alexey Ozerov
 */

defined('_JPI') or die('Direct access is not allowed');

require_once( JPATH_INSTALLATION.DS.'includes'.DS.'db.php' );

function removeInformationFromStorage()
{
	$storage =& JPStorage::getInstance();
	$storage->remove('start');
	$storage->remove('foffset');
	$storage->remove('totalqueries');
}

function throwError($message)
{
	// Render the AJAX response containing the error
	$ret = array('error' => $message);
	echo renderXML($ret);
	// Remove information from the storage
	removeInformationFromStorage();
}

// Make sure we are in RAW mode
$output =& JPOutput::getInstance();
$output->setMode('raw');

// Get data from the Storage
$storage =& JPStorage::getInstance();
$databases = $storage->get('databases');
$activeDatabase = $storage->get('activedatabase');

// Check that we have an active database and a list of databases to begin with
if(!is_array($databases) || !is_string($activeDatabase))
{
	throwError(JPText::_('ERROR_NODATABASE').' '.$activeDatabase);
	return;
}

// Load the active database parameter in the current namespace
extract($databases[$activeDatabase]);

// Parse request data, if any
$dbtype		= getParam('dbtype', $dbtype);
$dbhost		= getParam('dbhost', $dbhost);
$dbuser		= getParam('dbuser', $dbuser);
$dbpass		= getParam('dbpass', $dbpass);
$dbname		= getParam('dbname', $dbname);
$existing	= @getParam('existing', $existing);
$prefix		= getParam('prefix', $prefix);
$suppressfk = getParam('suppressfk', $suppressfk);
$maxchunk	= getParam('maxchunk', $maxchunk);
$maxqueries = getParam('maxqueries', $maxqueries);

// Make sure the up-to-date data are stored
$databases[$activeDatabase]['dbtype'] = $dbtype;
$databases[$activeDatabase]['dbhost'] = $dbhost;
$databases[$activeDatabase]['dbuser'] = $dbuser;
$databases[$activeDatabase]['dbpass'] = $dbpass;
$databases[$activeDatabase]['dbname'] = $dbname;
$databases[$activeDatabase]['existing'] = $existing;
$databases[$activeDatabase]['prefix'] = $prefix;
$databases[$activeDatabase]['suppressfk'] = $suppressfk;
$databases[$activeDatabase]['maxchunk'] = $maxchunk;
$databases[$activeDatabase]['maxqueries'] = $maxqueries;
$storage->set('databases', $databases);

define('DATA_CHUNK_LENGTH',	65536);			// How many bytes to read per step
define('MAX_QUERY_LINES',	300);			// How many lines may be considered to be one query (except text lines)
define('LINESPERSESSION',	$maxqueries);	// Maximum lines to be executed per one import session
define('BYTESPERSESSION',	$maxchunk);		// Maximum data to be restored per one import session

// Allowed comment delimiters: lines starting with these strings will be dropped by BigDump
$comment[] = '#';
$comment[] = '-- ';
$comment[]='---';
$comment[]='/*!';

header("Expires: Mon, 1 Dec 2003 01:00:00 GMT");
header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
header("Cache-Control: no-store, no-cache, must-revalidate");
header("Cache-Control: post-check=0, pre-check=0", false);
header("Pragma: no-cache");
header("MIME-Version: 1.0");
header("Content-Type: text/xml");

// Initialization
$file = false;

// Open the file
if (isset ($activeDatabase)) {
	$filename = JPATH_INSTALLATION.DS.'sql'.DS.$sqlfile;

	if ( !$file = @fopen($filename, "rt") ) {
		throwError(JPText::_('ERROR_CANTOPENDUMPFILE').' '.$sqlfile);
		return;
	}
	else
	{
		// Get the file size
		if (fseek($file, 0, SEEK_END) == 0) {
			$filesize = ftell($file);
		} else {
			throwError(JPText::_('ERROR_UNKNOWNFILESIZE'));
			return;
		}
	}
}

// *******************************************************************************************
// START IMPORT SESSION HERE
// *******************************************************************************************
$start = $storage->get('start',0);
$foffset = $storage->get('foffset', 0);

// Check start and foffset are numeric values
if (!is_numeric($start) || !is_numeric($foffset))
{
	throwError(JPText::_('ERROR_INVALIDPARAMETERS'));
	return;
}

$start = floor($start);
$foffset = floor($foffset);

// Check $foffset upon $filesize
if ($foffset > $filesize)
{
	throwError(JPText::_('ERROR_AFTEREOF'));
	return;
}

// Set file pointer to $foffset
if (fseek($file, $foffset) != 0)
{
	throwError(JPText::_('ERROR_CANTSETOFFSET'));
	return;
}

// Start processing queries from $file
$query = "";
$queries = 0;
$totalqueries = $storage->get('totalqueries',0);
$linenumber = $start;
$querylines = 0;
$inparents = false;
$totalsizeread = 0;

// Stay processing as long as the LINESPERSESSION is not reached or the query is still incomplete
while (($linenumber < ($start + LINESPERSESSION)) && ($totalsizeread <= BYTESPERSESSION)) {
	// Read the whole next line
	$dumpline = "";
	while (!feof($file) && substr($dumpline, -1) != "\n") {
		$dumpline .= fgets($file, DATA_CHUNK_LENGTH);
	}
	if ($dumpline === "")
		break;

	// Handle DOS and Mac encoded linebreaks (I don't know if it will work on Win32 or Mac Servers)
	$dumpline = str_replace("\r\n", "\n", $dumpline);
	$dumpline = str_replace("\r", "\n", $dumpline);

	// Skip comments and blank lines only if NOT in parents
	if (!$inparents) {
		$skipline = false;
		reset($comment);
		foreach ($comment as $comment_value) {
			if (!$inparents && (trim($dumpline) == "" || strpos($dumpline, $comment_value) === 0)) {
				$skipline = true;
				break;
			}
		}
		if ($skipline) {
			$linenumber++;
			continue;
		}
	}

	// Remove double back-slashes from the dumpline prior to count the quotes ('\\' can only be within strings)
	$dumpline_deslashed = str_replace("\\\\", "", $dumpline);

	// Count ' and \' in the dumpline to avoid query break within a text field ending by ;
	// Please don't use double quotes ('"')to surround strings, it wont work
	$parents = substr_count($dumpline_deslashed, "'") - substr_count($dumpline_deslashed, "\\'");
	if ($parents % 2 != 0)
	$inparents = !$inparents;

	// Add the line to query
	$query .= $dumpline;

	// Don't count the line if in parents (text fields may include unlimited linebreaks)
	if (!$inparents)
		$querylines++;

	// Stop if query contains more lines as defined by MAX_QUERY_LINES
	if ($querylines > MAX_QUERY_LINES) {
		throwError(JPText::_('ERROR_TOOMANYLINES'));
		return;
		break;
	}

	// Connect to the database server
	$db =& JPDatabase::getInstance($dbtype, $dbhost, $dbuser, $dbpass, $dbname, $prefix);

	if(!$db->connect())
	{
		throwError( JPText::_('ERROR_DBCONFAIL') );
		if( is_object($db) ) $db->disconnect();
		return;
	}

	// Execute query if end of query detected (; as last character) AND NOT in parents
	if( (substr(trim($dumpline),-1) == ';') && !$inparents ) {
	//if (ereg(";$", trim($dumpline)) && !$inparents) {
		if( ($queries == 0) )
		{
			// Enforce skipping foreign key checks, if specified
			if($suppressfk)
				$db->query('SET FOREIGN_KEY_CHECKS = 0');
		}

		// CREATE TABLE query pre-processing
		// If the table has a prefix, back it up (if requested). In any case, drop
		// the table. before attempting to create it.
		$replaceAll = false;
		if( substr($query, 0, 12) == 'CREATE TABLE')
		{
			// Yes, try to get the table name
			$restOfQuery = trim(substr($query, 12, strlen($query)-12 )); // Rest of query, after CREATE TABLE
			// Is there a backtick?
			if(substr($restOfQuery,0,1) == '`')
			{
				// There is... Good, we'll just find the matching backtick
				$pos = strpos($restOfQuery, '`', 1);
				$tableName = substr($restOfQuery,1,$pos - 1);
			}
			else
			{
				// Nope, let's assume the table name ends in the next blank character
				$pos = strpos($restOfQuery, ' ', 1);
				$tableName = substr($restOfQuery,1,$pos - 1);
			}
			unset($restOfQuery);

			// Should I back the table up?
			if( ($prefix != '') && ($existing == 'backup') && (strpos($tableName, '#__') == 0))
			{
				// It's a table with a prefix, a prefix IS specified and we are asked to back it up.
				// Start by dropping any existing backup tables
				$backupTable = str_replace('#__', 'bak_', $tableName);
				$dropQuery = 'DROP TABLE IF EXISTS `'.$backupTable.'`;';
				$db->setQuery(trim($dropQuery));
				if (!$db->query()) {
					// Query failure
					$message = "<p>".JPText::_('ERROR_DBERRATLINE')." $linenumber: ". trim($dumpline) . "</p>\n";
					$message .= "<p>".JPText::_('ERROR_LBLQUERY') .  trim(nl2br(htmlentities($dropQuery))) ."</p>\n";
					$message .="<p>MySQL: " . $db->getError() . "</p>\n";
					throwError($message);
					if( is_object($db) ) $db->disconnect();
					return;
					break;
				}

				// Then, rename the old table
				$renameQuery = 'ALTER TABLE `'.str_replace('#__', $prefix, $tableName).'` RENAME `'.$backupTable.'`;';
				$db->setQuery(trim($renameQuery));
				$db->query();
			}
			// Try to drop the table anyway
			$dropQuery = 'DROP TABLE IF EXISTS `'.$tableName.'`;';
			$db->setQuery(trim($dropQuery));
			if (!$db->query()) {
				// Query failure
				$message = "<p>".JPText::_('ERROR_DBERRATLINE')." $linenumber: ". trim($dumpline) . "</p>\n";
				$message .= "<p>".JPText::_('ERROR_LBLQUERY') .  trim(nl2br(htmlentities($dropQuery))) ."</p>\n";
				$message .="<p>MySQL: " . $db->getError() . "</p>\n";
				throwError($message);
				if( is_object($db) ) $db->disconnect();
				return;
				break;
			}
		} else
		// CREATE VIEW query pre-processing
		// In any case, drop the view before attempting to create it. (Views can't be renamed)
		if( (substr($query, 0, 7) == 'CREATE ') && (strpos($query, ' VIEW ') !== false) )
		{
			// Yes, try to get the view name
			$view_pos = strpos($query, ' VIEW ');
			$restOfQuery = trim( substr($query, $view_pos + 6) ); // Rest of query, after VIEW string
			// Is there a backtick?
			if(substr($restOfQuery,0,1) == '`')
			{
				// There is... Good, we'll just find the matching backtick
				$pos = strpos($restOfQuery, '`', 1);
				$tableName = substr($restOfQuery,1,$pos - 1);
			}
			else
			{
				// Nope, let's assume the table name ends in the next blank character
				$pos = strpos($restOfQuery, ' ', 1);
				$tableName = substr($restOfQuery,1,$pos - 1);
			}
			unset($restOfQuery);

			// Try to drop the view anyway
			$dropQuery = 'DROP VIEW IF EXISTS `'.$tableName.'`;';
			$db->setQuery(trim($dropQuery));
			if (!$db->query()) {
				// Query failure
				$message = "<p>".JPText::_('ERROR_DBERRATLINE')." $linenumber: ". trim($dumpline) . "</p>\n";
				$message .= "<p>".JPText::_('ERROR_LBLQUERY') .  trim(nl2br(htmlentities($dropQuery))) ."</p>\n";
				$message .="<p>MySQL: " . $db->getError() . "</p>\n";
				throwError($message);
				if( is_object($db) ) $db->disconnect();
				return;
				break;
			}

			$replaceAll = true; // When processing views, we might have to replace SEVERAL metaprefixes.
		}
		// CREATE PROCEDURE pre-processing
		elseif( (substr($query, 0, 7) == 'CREATE ') && (strpos($query, 'PROCEDURE ') !== false) )
		{
			// Try to get the procedure name
			$entity_keyword = ' PROCEDURE ';
			$entity_pos = strpos($query, $entity_keyword);
			$restOfQuery = trim( substr($query, $entity_pos + strlen($entity_keyword)) ); // Rest of query, after entity key string
			// Is there a backtick?
			if(substr($restOfQuery,0,1) == '`')
			{
				// There is... Good, we'll just find the matching backtick
				$pos = strpos($restOfQuery, '`', 1);
				$entity_name = substr($restOfQuery,1,$pos - 1);
			}
			else
			{
				// Nope, let's assume the entity name ends in the next blank character
				$pos = strpos($restOfQuery, ' ', 1);
				$entity_name = substr($restOfQuery,1,$pos - 1);
			}
			unset($restOfQuery);

			// Try to drop the entity anyway
			$dropQuery = 'DROP'.$entity_keyword.'IF EXISTS `'.$entity_name.'`;';
			$db->setQuery(trim($dropQuery));
			if (!$db->query()) {
				// Query failure
				$message = "<p>".JPText::_('ERROR_DBERRATLINE')." $linenumber: ". trim($dumpline) . "</p>\n";
				$message .= "<p>".JPText::_('ERROR_LBLQUERY') .  trim(nl2br(htmlentities($dropQuery))) ."</p>\n";
				$message .="<p>MySQL: " . $db->getError() . "</p>\n";
				throwError($message);
				if( is_object($db) ) $db->disconnect();
				return;
				break;
			}

			$replaceAll = true; // When processing entities, we might have to replace SEVERAL metaprefixes.
		}
		elseif( (substr($query, 0, 7) == 'CREATE ') && (strpos($query, 'FUNCTION ') !== false) )
		{
			// Try to get the procedure name
			$entity_keyword = ' FUNCTION ';
			$entity_pos = strpos($query, $entity_keyword);
			$restOfQuery = trim( substr($query, $entity_pos + strlen($entity_keyword)) ); // Rest of query, after entity key string
			// Is there a backtick?
			if(substr($restOfQuery,0,1) == '`')
			{
				// There is... Good, we'll just find the matching backtick
				$pos = strpos($restOfQuery, '`', 1);
				$entity_name = substr($restOfQuery,1,$pos - 1);
			}
			else
			{
				// Nope, let's assume the entity name ends in the next blank character
				$pos = strpos($restOfQuery, ' ', 1);
				$entity_name = substr($restOfQuery,1,$pos - 1);
			}
			unset($restOfQuery);

			// Try to drop the entity anyway
			$dropQuery = 'DROP'.$entity_keyword.'IF EXISTS `'.$entity_name.'`;';
			$db->setQuery(trim($dropQuery));
			if (!$db->query()) {
				// Query failure
				$message = "<p>".JPText::_('ERROR_DBERRATLINE')." $linenumber: ". trim($dumpline) . "</p>\n";
				$message .= "<p>".JPText::_('ERROR_LBLQUERY') .  trim(nl2br(htmlentities($dropQuery))) ."</p>\n";
				$message .="<p>MySQL: " . $db->getError() . "</p>\n";
				throwError($message);
				if( is_object($db) ) $db->disconnect();
				return;
				break;
			}

			$replaceAll = true; // When processing entities, we might have to replace SEVERAL metaprefixes.
		}
		elseif( (substr($query, 0, 7) == 'CREATE ') && (strpos($query, 'TRIGGER ') !== false) )
		{
			// Try to get the procedure name
			$entity_keyword = ' TRIGGER ';
			$entity_pos = strpos($query, $entity_keyword);
			$restOfQuery = trim( substr($query, $entity_pos + strlen($entity_keyword)) ); // Rest of query, after entity key string
			// Is there a backtick?
			if(substr($restOfQuery,0,1) == '`')
			{
				// There is... Good, we'll just find the matching backtick
				$pos = strpos($restOfQuery, '`', 1);
				$entity_name = substr($restOfQuery,1,$pos - 1);
			}
			else
			{
				// Nope, let's assume the entity name ends in the next blank character
				$pos = strpos($restOfQuery, ' ', 1);
				$entity_name = substr($restOfQuery,1,$pos - 1);
			}
			unset($restOfQuery);

			// Try to drop the entity anyway
			$dropQuery = 'DROP'.$entity_keyword.'IF EXISTS `'.$entity_name.'`;';
			$db->setQuery(trim($dropQuery));
			if (!$db->query()) {
				// Query failure
				$message = "<p>".JPText::_('ERROR_DBERRATLINE')." $linenumber: ". trim($dumpline) . "</p>\n";
				$message .= "<p>".JPText::_('ERROR_LBLQUERY') .  trim(nl2br(htmlentities($dropQuery))) ."</p>\n";
				$message .="<p>MySQL: " . $db->getError() . "</p>\n";
				throwError($message);
				if( is_object($db) ) $db->disconnect();
				return;
				break;
			}

			$replaceAll = true; // When processing entities, we might have to replace SEVERAL metaprefixes.
		}

		$query = trim($query);
		$query = rtrim($query, ';');
		if(!empty($query)) {
			$db->setQuery(trim($query), !$replaceAll);
			if (!$db->query()) {
				// Query failure
				$message = "<p>".JPText::_('ERROR_DBERRATLINE')." $linenumber: ". trim($dumpline) . "</p>\n";
				$message .= "<p>".JPText::_('ERROR_LBLQUERY') .  trim(nl2br(htmlentities($query))) ."</p>\n";
				$message .="<p>MySQL: " . $db->getError() . "</p>\n";
				throwError($message);
				if( is_object($db) ) $db->disconnect();
				return;
				break;
			}
		}

		$totalsizeread += strlen($query);
		$totalqueries++;
		$queries++;
		$query = "";
		$querylines = 0;
	}
	$linenumber++;
}

// Get the current file position

$foffset = ftell($file);
if (!$foffset) {
	throwError(JPText::_('ERROR_CANTREADPOINTER'));
	if( is_object($db) ) $db->disconnect();
	return;
}

// Return statistics
$lines_this = $linenumber - $start;
$lines_done = $linenumber -1;

$queries_this = $queries;
$queries_done = $totalqueries;

$bytes_this = $foffset - $foffset;
$bytes_done = $foffset;
$kbytes_this = round($bytes_this / 1024, 2);
$kbytes_done = round($bytes_done / 1024, 2);
$mbytes_this = round($kbytes_this / 1024, 2);
$mbytes_done = round($kbytes_done / 1024, 2);

$bytes_togo = $filesize - $foffset;
$bytes_tota = $filesize;
$kbytes_togo = round($bytes_togo / 1024, 2);
$kbytes_tota = round($bytes_tota / 1024, 2);
$mbytes_togo = round($kbytes_togo / 1024, 2);
$mbytes_tota = round($kbytes_tota / 1024, 2);

$pct_this = ceil($bytes_this / $filesize * 100);
$pct_done = ceil($foffset / $filesize * 100);
$pct_togo = 100 - $pct_done;
$pct_tota = 100;

if ($bytes_togo == 0) {
	// Clear stored variables if we're finished
	$lines_togo = '0';
	$lines_tota = $linenumber -1;
	$queries_togo = '0';
	$queries_tota = $totalqueries;
	removeInformationFromStorage();
}
else
{
	// Save variables in storage!
	$storage->set('start', $start);
	$storage->set('foffset', $foffset);
	$storage->set('totalqueries', $totalqueries);
}

// Close the file
if ($file)
	fclose($file);

// Disconnect the database
$db->disconnect();

// Return meaningful data to AJAX
$ret = array(
	'percent'			=> $pct_done,
	'message'			=> sprintf(JPText::_('LBL_PROGRESS'), $kbytes_done, $kbytes_tota, $pct_done),
	'error'				=> '',
	'done'				=> ($bytes_togo == 0) ? '1' : '0'
);
echo renderXML($ret);